==> entity_theme_engine-8.x-1.1.tar.gz: entity_theme_engine/src/Normalizer/ContentEntityNormalizer.php <==
<?php

namespace Drupal\entity_theme_engine\Normalizer;

use Drupal\serialization\Normalizer\NormalizerBase;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\file\Entity\File;
use Drupal\Core\Cache\Cache;

class ContentEntityNormalizer extends NormalizerBase {

  /**
   * @var string[]
   */
  protected $supportedInterfaceOrClass = ['Drupal\Core\Entity\ContentEntityInterface'];

  /**
   * @var string[]
   */
  protected $format = ['twig_variable'];

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;
  
  /**
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public function normalize($entity, $format = NULL, array $context = []) {
    $data = [
      'entity_id' => $entity->id(),
      'entity_uuid' => $entity->uuid(),
      'entity_label' => $entity->label(),
      'entity_url' => $entity->hasLinkTemplate('canonical')?$entity->toUrl()->toString():"",
      'entity_type' => $entity->getEntityTypeId(),
    ];
    $cache_tags = ["{$entity->getEntityTypeId()}:{$entity->id()}"];
    if($entity instanceof FieldableEntityInterface) {
      $fields = $entity->getFieldDefinitions();
      foreach ($fields as $field_name => $field_def) {
        $field_type = $field_def->getType();
        $sub_context = $context + [
        'field_name' => $field_name,
        'field_type' => $field_type,
        'field_definition' => $field_def
        ];
        if($field_def->getFieldStorageDefinition()->getCardinality() == 1) {
          if($entity->get($field_name)->isEmpty()){
            $data[$field_name] = NULL;
          }else {
            $field_data = $this->serializer->serialize($entity->get($field_name)->get(0), $format, $sub_context);
            if(!empty($field_data['#cache_tags'])) {
              $cache_tags = Cache::mergeTags($cache_tags,$field_data['#cache_tags']);
            }
            $data[$field_name] = $field_data;
          }
        }else {
          foreach ($entity->get($field_name) as $item) {
            $field_data = $this->serializer->serialize($item, $format, $sub_context);
            if(!empty($field_data['#cache_tags'])) {
              $cache_tags = Cache::mergeTags($cache_tags,$field_data['#cache_tags']);
            }
            $data[$field_name]['items'][] = $field_data;
          }
        }
      }
    }
    $data['#cache_tags'] = $cache_tags;
    return $data;
  }
  /**
   * {@inheritdoc}
   */
  public function supportsNormalization($data, $format = NULL) {
    if(in_array($format, $this->format) && parent::supportsNormalization($data, $format)) {
      return TRUE;
    }
    return FALSE;
  }

  public function getImageStylesVariables(File $entity) {
    $variables = [];
    $styles = static::getStyles();
    foreach ($styles as $id => $style) {
      $variables[$id] = $style->buildUrl($entity->getFileUri());
    }
    return $variables;
  }
  public function getStyles() {
    $cache = &drupal_static(__METHOD__, []);
    if(empty($cache)) {
      $cache = $this->entityTypeManager->getStorage('image_style')->loadMultiple();
    }
    return $cache;
  }
}

==> entity_theme_engine-8.x-1.2.tar.gz: entity_theme_engine/src/Normalizer/ContentEntityNormalizer.php <==
<?php

namespace Drupal\entity_theme_engine\Normalizer;

use Drupal\serialization\Normalizer\NormalizerBase;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\file\Entity\File;
use Drupal\Core\Cache\Cache;

class ContentEntityNormalizer extends NormalizerBase {

  /**
   * @var string[]
   */
  protected $supportedInterfaceOrClass = ['Drupal\Core\Entity\ContentEntityInterface'];

  /**
   * @var string[]
   */
  protected $format = ['twig_variable'];

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;
  
  /**
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public function normalize($entity, $format = NULL, array $context = []) {
    $data = [
      'entity_id' => $entity->id(),
      'entity_uuid' => $entity->uuid(),
      'entity_label' => $entity->label(),
      'entity_url' => ($entity->id() && $entity->hasLinkTemplate('canonical'))?$entity->toUrl()->toString():"",
      'entity_type' => $entity->getEntityTypeId(),
    ];
    $cache_tags = ["{$entity->getEntityTypeId()}:{$entity->id()}"];
    if($entity instanceof FieldableEntityInterface) {
      $fields = $entity->getFieldDefinitions();
      foreach ($fields as $field_name => $field_def) {
        $field_type = $field_def->getType();
        $sub_context = $context + [
        'field_name' => $field_name,
        'field_type' => $field_type,
        'field_definition' => $field_def
        ];
        if($field_def->getFieldStorageDefinition()->getCardinality() == 1) {
          if($entity->get($field_name)->isEmpty()){
            $data[$field_name] = NULL;
          }else {
            $field_data = $this->serializer->serialize($entity->get($field_name)->get(0), $format, $sub_context);
            if(!empty($field_data['#cache_tags'])) {
              $cache_tags = Cache::mergeTags($cache_tags,$field_data['#cache_tags']);
            }
            $data[$field_name] = $field_data;
          }
        }else {
          foreach ($entity->get($field_name) as $item) {
            $field_data = $this->serializer->serialize($item, $format, $sub_context);
            if(!empty($field_data['#cache_tags'])) {
              $cache_tags = Cache::mergeTags($cache_tags,$field_data['#cache_tags']);
            }
            $data[$field_name]['items'][] = $field_data;
          }
        }
      }
    }
    $data['#cache_tags'] = $cache_tags;
    return $data;
  }
  /**
   * {@inheritdoc}
   */
  public function supportsNormalization($data, $format = NULL) {
    if(in_array($format, $this->format) && parent::supportsNormalization($data, $format)) {
      return TRUE;
    }
    return FALSE;
  }

  public function getImageStylesVariables(File $entity) {
    $variables = [];
    $styles = static::getStyles();
    foreach ($styles as $id => $style) {
      $variables[$id] = $style->buildUrl($entity->getFileUri());
    }
    return $variables;
  }
  public function getStyles() {
    $cache = &drupal_static(__METHOD__, []);
    if(empty($cache)) {
      $cache = $this->entityTypeManager->getStorage('image_style')->loadMultiple();
    }
    return $cache;
  }
}
